@@ -39,7 +39,7 @@ static int getLine (int fd, char *s, int len)
   char ch;
   int bytes = 0;
 
-  while (read (fd, &ch, 1) != -1)
+  while (read (fd, &ch, 1) > 0)
   {
     *s++ = ch;
     bytes++;
@@ -63,7 +63,7 @@ static int getPass (void)
 {
   if (!PopPass[0])
   {
-    if (mutt_get_password ("POP Password: ", PopPass, sizeof(PopPass)) != 0)
+    if (mutt_get_password ("POP Password: ", PopPass, sizeof (PopPass)) != 0)
       return 0;
   }
   return 1;
@@ -79,28 +79,29 @@ void mutt_fetchPopMail (void)
 #endif
   struct hostent *he;
   char buffer[2048];
+  char msgbuf[SHORT_STRING];
   int s, i, msgs, bytes, err = 0;
   time_t now;
-  FOLDER *fol;
+  CONTEXT ctx;
+  MESSAGE *msg = NULL;
 
   if (!PopHost[0])
   {
-    mutt_error("POP host is not defined.");
+    mutt_error ("POP host is not defined.");
     return;
   }
 
   if (!PopUser[0])
   {
-    mutt_error("No POP username is defined.");
+    mutt_error ("No POP username is defined.");
     return;
   }
     
-  if (!getPass())
-    return;
+  if (!getPass ()) return;
 
   s = socket (AF_INET, SOCK_STREAM, IPPROTO_IP);
 
-  memset ((char *)&sin, 0, sizeof(sin));
+  memset ((char *) &sin, 0, sizeof(sin));
   sin.sin_family = AF_INET;
   sin.sin_port = htons (PopPort);
 
@@ -117,49 +118,60 @@ void mutt_fetchPopMail (void)
   else
     memcpy ((void *)&sin.sin_addr, (void *)&n, sizeof(n));
   
-  mutt_error ("Connecting to %s", inet_ntoa(sin.sin_addr));
+  mutt_message ("Connecting to %s", inet_ntoa (sin.sin_addr));
 
-  if (connect (s, (struct sockaddr *)&sin, sizeof (struct sockaddr_in)) == -1)
+  if (connect (s, (struct sockaddr *) &sin, sizeof (struct sockaddr_in)) == -1)
   {
     mutt_perror ("connect");
     return;
   }
   
-  getLine (s, buffer, sizeof(buffer));
+  if (getLine (s, buffer, sizeof (buffer)) == -1)
+    goto fail;
+
   if (strncmp (buffer, "+OK", 3) != 0)
   {
+    mutt_remove_trailing_ws (buffer);
     mutt_error (buffer);
     goto finish;
   }
 
   sprintf (buffer, "user %s\r\n", PopUser);
-  write (s, buffer, strlen(buffer));
+  write (s, buffer, strlen (buffer));
+
+  if (getLine (s, buffer, sizeof (buffer)) == -1)
+    goto fail;
 
-  getLine (s, buffer, sizeof(buffer));
   if (strncmp (buffer, "+OK", 3) != 0)
   {
+    mutt_remove_trailing_ws (buffer);
     mutt_error (buffer);
     goto finish;
   }
   
   sprintf (buffer, "pass %s\r\n", PopPass);
-  write (s, buffer, strlen(buffer));
+  write (s, buffer, strlen (buffer));
   
-  getLine (s, buffer, sizeof(buffer));
+  if (getLine (s, buffer, sizeof (buffer)) == -1)
+    goto fail;
+
   if (strncmp (buffer, "+OK", 3) != 0)
   {
     PopPass[0] = 0; /* void the given password */
-    mutt_error (buffer);
+    mutt_remove_trailing_ws (buffer);
+    mutt_error (buffer[0] ? buffer : "Server closed connection!");
     goto finish;
   }
   
   /* find out how many messages are in the mailbox. */
-  strcpy (buffer, "stat\r\n");
-  write (s, buffer, strlen (buffer));
+  write (s, "stat\r\n", 6);
   
-  getLine (s, buffer, sizeof (buffer));
-  if (strncmp(buffer, "+OK", 3) != 0)
+  if (getLine (s, buffer, sizeof (buffer)) == -1)
+    goto fail;
+
+  if (strncmp (buffer, "+OK", 3) != 0)
   {
+    mutt_remove_trailing_ws (buffer);
     mutt_error (buffer);
     goto finish;
   }
@@ -168,54 +180,45 @@ void mutt_fetchPopMail (void)
 
   if (msgs == 0)
   {
-    mutt_error ("No new mail in POP mailbox.");
+    mutt_message ("No new mail in POP mailbox.");
     goto finish;
   }
 
-  strfcpy (buffer, Spoolfile, sizeof (buffer));
-  mutt_expand_path (buffer, sizeof (buffer));
-  if ((fol = folder_open (buffer, "a")) == NULL)
+  if (mx_open_mailbox (Spoolfile, M_APPEND, &ctx) == NULL)
     goto finish;
 
-#ifdef USE_SETGID
-  if (SETEGID (MailGid) != 0)
-  {
-    mutt_perror ("setegid");
-    folder_close (&fol);
-    goto finish;
-  }
-#endif /* USE_SETGID */
+  snprintf (msgbuf, sizeof (msgbuf),
+	    "Reading %d new message%s (%d bytes)...", msgs, msgs > 1 ? "s" : "", bytes);
+  mutt_message (msgbuf);
 
-  if (lock_file (fol, 1) != 0)
-  {
-#ifdef USE_SETGID
-    SETEGID (UserGid);
-#endif
-
-    mutt_error ("Unable to lock mailbox!");
-    folder_close (&fol);
-    goto finish;
-  }
-
-  mutt_error ("Reading %d new message%s (%d bytes)...",
-	      msgs, msgs > 1 ? "s" : "", bytes);
-
-  fseek (fol->fp, 0, 2);
-  
   for (i = 1 ; i <= msgs ; i++)
   {
     sprintf (buffer, "retr %d\r\n", i);
     write (s, buffer, strlen (buffer));
 
-    getLine(s, buffer, sizeof(buffer));
+    if (getLine (s, buffer, sizeof (buffer)) == -1)
+    {
+      mx_fastclose_mailbox (&ctx);
+      goto fail;
+    }
+
     if (strncmp (buffer, "+OK", 3) != 0)
     {
+      mutt_remove_trailing_ws (buffer);
       mutt_error (buffer);
       break;
     }
 
+    if ((msg = mx_open_new_message (&ctx, NULL)) == NULL)
+    {
+      err = 1;
+      break;
+    }
+
     now = time (0);
-    fprintf (fol->fp, "From %s %s", Username, ctime (&now));
+    
+    if (msg->magic == M_MMDF || msg->magic == M_MBOX)
+      fprintf (msg->fp, "From %s %s", Username, ctime (&now));
 
     /* Now read the actual message. */
     FOREVER
@@ -223,9 +226,9 @@ void mutt_fetchPopMail (void)
       char *p;
       int chunk;
 
-      if ((chunk = getLine(s, buffer, sizeof(buffer))) == -1)
+      if ((chunk = getLine (s, buffer, sizeof (buffer))) == -1)
       {
-	mutt_error("Error reading message!");
+	mutt_error ("Error reading message!");
 	err = 1;
 	break;
       }
@@ -256,10 +259,10 @@ void mutt_fetchPopMail (void)
       else
 	p = buffer;
       
-      fwrite (p, 1, chunk, fol->fp);
+      fwrite (p, 1, chunk, msg->fp);
     }
 
-    if (ferror (fol->fp))
+    if (mx_close_message (&msg) != 0)
     {
       mutt_error ("Error while writing mailbox!");
       err = 1;
@@ -279,22 +282,18 @@ void mutt_fetchPopMail (void)
       if (strncmp (buffer, "+OK", 3) != 0)
       {
 	err = 1;
+        mutt_remove_trailing_ws (buffer);
 	mutt_error (buffer);
 	break;
       }
     }
+
+    mutt_message ("%s [%d messages read]", msgbuf, i);
   }
 
-  if (fflush (fol->fp) != 0)
-    err = 1;
-  
-  unlock_file (fol);
-
-#ifdef USE_SETGID
-  SETEGID (UserGid);
-#endif
-
-  folder_close (&fol);
+  if (msg)
+    mx_close_message (&msg);
+  mx_close_mailbox (&ctx);
 
   if (err)
   {
@@ -309,8 +308,14 @@ finish:
   write (s, "quit\r\n", 6);
   getLine (s, buffer, sizeof (buffer)); /* snarf the response */
   close (s);
-
   return;
+
+  /* not reached */
+
+fail:
+
+  mutt_error ("Server closed connection!");
+  close (s);
 }
 
 #endif /* USE_POP */
