@@ -16,31 +16,27 @@
  *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */ 
 
-#include "config.h"
+#include "mutt.h"
+#include "mutt_curses.h"
 
-#ifdef USE_SLANG_CURSES
-
-#include <termios.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <termios.h>
 
-#if defined(__linux__) || defined(_AIX) || defined(__osf__)
+#ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
 #endif
 
-#ifndef STDC_HEADERS
-extern int ioctl ();
-#endif
-
-#include "slcurses.h"
-
 /* this routine should be called after receiving SIGWINCH */
 void mutt_resize_screen (void)
 {
   char *cp;
   int fd;
   struct winsize w;
+#ifdef HAVE_RESIZETERM
+  int SLtt_Screen_Rows, SLtt_Screen_Cols;
+#endif
 
   SLtt_Screen_Rows = -1;
   SLtt_Screen_Cols = -1;
@@ -69,11 +65,13 @@ void mutt_resize_screen (void)
     else
       SLtt_Screen_Cols = 80;
   }
+#ifdef USE_SLANG_CURSES
   delwin (stdscr);
   SLsmg_reset_smg ();
   SLsmg_init_smg ();
-  stdscr = newwin (0,0,0,0);
+  stdscr = newwin (0, 0, 0, 0);
   keypad (stdscr, TRUE);
+#else
+  resizeterm (SLtt_Screen_Rows, SLtt_Screen_Cols);
+#endif
 }
-
-#endif /* USE_SLANG_CURSES */
